
function [MY,ME,ETA,AMU_,ne,r2,RC] = model_solution_rank_ind(para)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% For given parameter values, this function 
% 1) puts the model in Gensys' canonical form
% 2) solves the RE system of equations using Chris Sims' Gensys

% Canonical form: A0*x(t) = A1*x(t-1) + B*e_t + C*eta_t
% The solution takes the form:  x(t) = MY * x(t-1) + ME * e(t)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% -------------------------------------------------------------------------
% Parameters
% -------------------------------------------------------------------------

sig_b          = para(1);           % st dev of shocks
sig_i          = para(2);
sig_r          = para(3);
sig_p          = para(4);
sig_w          = para(5);
sig_g          = para(6);
sig_a          = para(7);
sig_v1         = para(8);
rho_pv1        = para(9);


phiPI   = para(10);        % response to inflation
phiY    = para(11);        % response to output
phiGY   = para(12);        % response to output growth
phiR    = para(13);        % TR smoothing
phi_l   = para(14);        % invserse Frish elasticity of labor supply
b       = para(15);        % habit
gamI    = para(16);        % investment adjust cost
ksi_p   = para(17);        % calvo price stickiness
ksi_w   = para(18);        % calvo wage stickiness
chi_p   = para(19);       % price indexation
chi_w   = para(20);       % wage indexation
sigma_u = para(21);       % elasticity of the capital utilization cost function
sigma   = para(22);       % elasticity of inter-temporal substitution, sigma = 1 implies log utility
alpha   = para(23);       % share of capital in the prod. function
gzstar  = para(24);       % quarterly net growth rate in steady state
Hss     = para(25);          % steady state for log hours
pss100  = para(26);          % quarterly net growth rate of inflation

rho_b          = para(27);         % AR(1) coefficients of the shock processes
rho_i          = para(28);
rho_r          = para(29);
rho_lambda_p   = para(30);
rho_lambda_w   = para(31);
rho_g          = para(32);
rho_a          = para(33);
cmap           = para(34);
cmaw           = para(35);
rho_gy         = para(36);




beta       = 0.9975;           % discount factor
delta      = 0.025;            % depreciation rate
alpha_p    = 6;                % goods elasticity of demand
lambda_p   = 0.2;              % steady state net price markup
gz         = 1+(gzstar/100);   % quarterly gross growth rate in steady state
g_y        = 0.20;             % gov spending to GDP ration
crt_co     = 1;                % steady state consumption ratio between two groups
theta      = 0;                % fraction of HTMC
alpha1     = 1.5;              % AR(1) coefficient of the augmented equation ala Bianchi-Nicolo

% -------------------------------------------------------------------------
% steady state relationships
% -------------------------------------------------------------------------

rk = (gz/beta)+delta-1;
i_k = 1 - ((1-delta)/gz);
w = ((1/(1+lambda_p))*(((alpha^(alpha))*((1-alpha)^(1-alpha)))/(rk^alpha)))^(1/(1-alpha));
h_k = (1/gz)*((1-alpha)/alpha)*(rk/w);
k_y = (1+lambda_p)*(gz^alpha)*(h_k^(alpha-1));
i_y = i_k*k_y;
c_y = 1-i_y-g_y;
wh_c = (1-alpha)*(1/c_y);
c_rt = ((1/(1-theta))*crt_co)/(1+(theta/(1-theta))*crt_co);
c_o = (1/(1-theta)) - (theta/(1-theta))*c_rt;
omega = (theta/(1-theta))*(((c_rt-b)/(c_o-b))^(-sigma));
A = 1/(lambda_p*alpha_p+1);



%=========================================================================
%                        SOLVE DSGE MODEL
%=========================================================================



%=========================================================================
%                       DEFINE OBJECTS
%=========================================================================

% Equation indices


eq_1 = 1;
eq_2 = 2;
eq_3 = 3;
eq_4 = 4;
eq_5 = 5;
eq_6 = 6;
eq_7 = 7;
eq_8 = 8;
eq_9 = 9;
eq_10 = 10;
eq_11 = 11;
eq_12 = 12;
eq_13 = 13;
eq_14 = 14;
eq_15 = 15;
eq_16 = 16;
eq_17 = 17;
eq_18 = 18;
eq_19 = 19;
eq_20 = 20;
eq_21 = 21;
eq_22 = 22;
eq_23 = 23;
eq_24 = 24;
eq_25 = 25;
eq_26 = 26;
eq_27 = 27;
eq_28 = 28;
eq_29 = 29;
eq_30 = 30;
eq_31 = 31;
eq_32 = 32;
eq_33 = 33;
eq_34 = 34;
eq_35 = 35;
eq_36 = 36;
eq_37 = 37;
eq_38 = 38;
eq_39 = 39;
eq_40 = 40;
eq_41 = 41;
eq_42 = 42;
eq_43 = 43;
eq_44 = 44;
eq_45 = 45;
eq_46 = 46;
eq_47 = 47;
eq_48 = 48;
eq_49 = 49;
eq_50 = 50;
eq_51 = 51;
eq_52 = 52;
eq_53 = 53;
eq_54 = 54;



% -------------------------------------------------------------------------
% INDEX for endogenous variables
% -------------------------------------------------------------------------

c_o_t = 1;        % consumption of ricardian agents
c_t = 2;          % aggregate consumption
R_t = 3;          % Nominal interest rate
i_t = 4;          % Investment
Q_o_t = 5;        % Lagrange multiplier (Shadow price of capital)
r_k_t = 6;        % rental rate of capital
u_t = 7;          % utilization rate
k_t = 8;          % capital stock, k_t+1
c_rt_t = 9;       % consumption of non-ricardian agents
w_t = 10;         % real wage
h_t = 11;         % hours
g_t = 12;         % exogenous gov spending
pi_t = 13;        % inflation
y_t = 14;         % aggregate output
mc_t = 15;        % marginal cost
lambda_p_t = 16;  % price markup shock
mrs_o_t = 17;     % marginal rate of substitution (ricardian)
mrs_rt_t = 18;    % marginal rate of substitution (non-ricardian)
lambda_w_t = 19;  % wage markup shock
gy_t = 20;        % growth rate of output
gc_t = 21;        % growth rate of consumption
gi_t = 22;        % growth rate of investment
gw_t = 23;        % growth rate of wage

Ec_o_t = 24;      % expectational variable
Epi_t = 25;
Ei_t = 26;
Er_k_t = 27;
EQ_o_t = 28;
Ew_t = 29;

eps_a_t = 30;     % exogenous shocks
eps_b_t = 31;
eps_i_t = 32;
eps_r_t = 33;
eps_p_t = 34;
eps_w_t = 35;


c_o_t_f = 36;        
c_t_f = 37;          
R_t_f = 38;          
i_t_f = 39;          
Q_o_t_f = 40;               
r_k_t_f = 41;        
u_t_f = 42;          
k_t_f = 43;          
c_rt_t_f = 44;      
w_t_f = 45;         
h_t_f = 46;                           
y_t_f = 47;       
mrs_o_t_f = 48;     
mrs_rt_t_f = 49;    

Ec_o_t_f = 50;      
Ei_t_f = 51;
Er_k_t_f = 52;
EQ_o_t_f = 53;

omega1_t = 54;


% -------------------------------------------------------------------------
% Expectation error indices (eta)
% -------------------------------------------------------------------------

eco_sh  = 1;
epi_sh  = 2;
ei_sh  = 3;
erk_sh  = 4;
eQ_sh  = 5;
ew_sh = 6;
eco_sh_f  = 7;
ei_sh_f  = 8;
erk_sh_f  = 9;
eQ_sh_f  = 10;


% -------------------------------------------------------------------------
% Shock indices (eps)
% ------------------------------------------------------------------------

a_sh = 1;
b_sh = 2;
i_sh = 3;
r_sh = 4;
p_sh = 5;
w_sh = 6;
g_sh = 7;
v1_sh = 8;


% -------------------------------------------------------------------------
% Summary
% ------------------------------------------------------------------------


neq  = 54;
neta = 10;
neps = 8;

% -------------------------------------------------------------------------
% initialize matrices
% ------------------------------------------------------------------------

A0 = zeros(neq,neq);
A1 = zeros(neq,neq);
B = zeros(neq,neps);
C = zeros(neq,neta);

%=========================================================================
%                EQUILIBRIUM CONDITIONS: CANONICAL SYSTEM
%=========================================================================


A0(eq_1,c_o_t) = 1;
A0(eq_1,Ec_o_t) = -1;
A0(eq_1,c_t) = ((1-b*(1/c_o))*b)/(c_o-b);
A0(eq_1,R_t) = (1-b*(1/c_o))/sigma;
A0(eq_1,Epi_t) = -(1-b*(1/c_o))/sigma;
A0(eq_1,eps_b_t) = 1;
A1(eq_1,c_t) = ((1-b*(1/c_o))*b)/(c_o-b);

A0(eq_2,i_t) = 1;
A0(eq_2,Q_o_t) = -(1/(gamI*(gz^2)*(1+beta)));
A0(eq_2,Ei_t) = -(beta/(1+beta));
A0(eq_2,eps_i_t) = -1;
A1(eq_2,i_t) = (1/(1+beta));

A0(eq_3,R_t) = -1;
A0(eq_3,eps_b_t) = -(sigma/(1-b*(1/c_o)));
A0(eq_3,Epi_t) = 1;
A0(eq_3,Er_k_t) = (beta/gz)*rk;
A0(eq_3,EQ_o_t) = (beta/gz)*(1-delta);
A0(eq_3,Q_o_t) = -1;

A0(eq_4,r_k_t) = 1;
A0(eq_4,u_t) = -(sigma_u/(1-sigma_u));

A0(eq_5,k_t) = 1;
A0(eq_5,i_t) = -i_k;
A0(eq_5,eps_i_t) = -i_k*gamI*(gz^2)*(1+beta);
A1(eq_5,k_t) = (1-delta)/gz;

A0(eq_6,c_rt_t) = c_rt;
A0(eq_6,w_t) = -wh_c;
A0(eq_6,h_t) = -wh_c;

A0(eq_7,c_t) = c_y;
A0(eq_7,g_t) = 1;
A0(eq_7,i_t) = i_y;
A0(eq_7,y_t) = -1;
A0(eq_7,u_t) = (rk/gz)*k_y;

A0(eq_8,c_t) = 1;
A0(eq_8,c_rt_t) = -theta*c_rt;
A0(eq_8,c_o_t) = -(1-theta)*c_o;

A0(eq_9,pi_t) = 1+beta*chi_p;
A0(eq_9,Epi_t) = -beta;
A0(eq_9,mc_t) = - A*(((1-beta*ksi_p)*(1-ksi_p))/ksi_p);
A0(eq_9,lambda_p_t) = -1;
A1(eq_9,pi_t) = chi_p;

A0(eq_10,w_t) = ((1+beta)*ksi_w + (1-ksi_w)*(1-ksi_w*beta))/((1+beta)*ksi_w);
A0(eq_10,mrs_o_t) = -(((1-ksi_w)*(1-ksi_w*beta))/((1+beta)*ksi_w))*(1/(1+omega));
A0(eq_10,mrs_rt_t) = -(((1-ksi_w)*(1-ksi_w*beta))/((1+beta)*ksi_w))*(omega/(1+omega));
A0(eq_10,Ew_t) = -(beta/(1+beta));
A0(eq_10,pi_t) = (1+beta*chi_w)/(1+beta);
A0(eq_10,Epi_t) = -(beta/(1+beta));
A0(eq_10,lambda_w_t) = -1;
A1(eq_10,w_t) = (1/(1+beta));
A1(eq_10,pi_t) = (chi_w/(1+beta));

A0(eq_11,mrs_o_t) = 1;
A0(eq_11,c_o_t) = -(sigma/(1-b*(1/c_o)));
A0(eq_11,h_t) = -phi_l;
A1(eq_11,c_t) = -((b*sigma)/(c_o-b));

A0(eq_12,mrs_rt_t) = 1;
A0(eq_12,c_rt_t) = -(sigma/(1-b*(1/c_rt)));
A0(eq_12,h_t) = -phi_l;
A1(eq_12,c_t) = -((b*sigma)/(c_rt-b));

A0(eq_13,u_t) = 1;
A0(eq_13,h_t) = -1;
A0(eq_13,w_t) = -1;
A0(eq_13,r_k_t) = 1;
A1(eq_13,k_t) = -1;

A0(eq_14,mc_t) = 1;
A0(eq_14,eps_a_t) = 1;
A0(eq_14,r_k_t) = -alpha;
A0(eq_14,w_t) = -(1-alpha);

A0(eq_15,y_t) = 1;
A0(eq_15,eps_a_t) = -(1+lambda_p);
A0(eq_15,u_t) = -(1+lambda_p)*alpha;
A0(eq_15,h_t) = -(1+lambda_p)*(1-alpha);
A1(eq_15,k_t) = (1+lambda_p)*alpha;

A0(eq_16,R_t) = 1;
A0(eq_16,pi_t) = -(1-phiR)*phiPI;
A0(eq_16,y_t) = -((1-phiR)*phiY+phiGY);
A0(eq_16,y_t_f) = ((1-phiR)*phiY+phiGY);
A0(eq_16,eps_r_t) = -1;
A1(eq_16,R_t) = phiR;
A1(eq_16,y_t) = -phiGY;
A1(eq_16,y_t_f) = phiGY;

A0(eq_17,gy_t) = 1;
A0(eq_17,y_t) = -1;
A1(eq_17,y_t) = -1;

A0(eq_18,c_o_t) = 1;
A1(eq_18,Ec_o_t) = 1;
C(eq_18,eco_sh) = 1;

A0(eq_19,pi_t) = 1;
A1(eq_19,Epi_t) = 1;
C(eq_19,epi_sh) = 1;

A0(eq_20,i_t) = 1;
A1(eq_20,Ei_t) = 1;
C(eq_20,ei_sh) = 1;

A0(eq_21,r_k_t) = 1;
A1(eq_21,Er_k_t) = 1;
C(eq_21,erk_sh) = 1;

A0(eq_22,Q_o_t) = 1;
A1(eq_22,EQ_o_t) = 1;
C(eq_22,eQ_sh) = 1;

A0(eq_23,w_t) = 1;
A1(eq_23,Ew_t) = 1;
C(eq_23,ew_sh) = 1;

A0(eq_24,eps_a_t) = 1;
A1(eq_24,eps_a_t) = rho_a;
B(eq_24,a_sh) = 1;

A0(eq_25,eps_b_t) = 1;
A1(eq_25,eps_b_t) = rho_b;
B(eq_25,b_sh) = 1;

A0(eq_26,eps_i_t) = 1;
A1(eq_26,eps_i_t) = rho_i;
B(eq_26,i_sh) = 1;

A0(eq_27,eps_r_t) = 1;
A1(eq_27,eps_r_t) = rho_r;
B(eq_27,r_sh) = 1;

A0(eq_28,lambda_p_t) = 1;
A0(eq_28,eps_p_t) = -1;
A1(eq_28,lambda_p_t) = rho_lambda_p;
A1(eq_28,eps_p_t) = -cmap;

A0(eq_29,eps_p_t) = 1;
B(eq_29,p_sh) = 1;

A0(eq_30,lambda_w_t) = 1;
A0(eq_30,eps_w_t) = -1;
A1(eq_30,lambda_w_t) = rho_lambda_w;
A1(eq_30,eps_w_t) = -cmaw;

A0(eq_31,eps_w_t) = 1;
B(eq_31,w_sh) = 1;

A0(eq_32,g_t) = 1;
A1(eq_32,g_t) = rho_g;
B(eq_32,g_sh) = 1;
B(eq_32,a_sh) = rho_gy;


A0(eq_33,c_o_t_f) = 1;
A0(eq_33,Ec_o_t_f) = -1;
A0(eq_33,c_t_f) = ((1-b*(1/c_o))*b)/(c_o-b);
A0(eq_33,R_t_f) = (1-b*(1/c_o))/sigma;
A0(eq_33,eps_b_t) = 1;
A1(eq_33,c_t_f) = ((1-b*(1/c_o))*b)/(c_o-b);

A0(eq_34,i_t_f) = 1;
A0(eq_34,Q_o_t_f) = -(1/(gamI*(gz^2)*(1+beta)));
A0(eq_34,Ei_t_f) = -(beta/(1+beta));
A0(eq_34,eps_i_t) = -1;
A1(eq_34,i_t_f) = (1/(1+beta));

A0(eq_35,R_t_f) = -1;
A0(eq_35,eps_b_t) = -(sigma/(1-b*(1/c_o)));
A0(eq_35,Er_k_t_f) = (beta/gz)*rk;
A0(eq_35,EQ_o_t_f) = (beta/gz)*(1-delta);
A0(eq_35,Q_o_t_f) = -1;

A0(eq_36,r_k_t_f) = 1;
A0(eq_36,u_t_f) = -(sigma_u/(1-sigma_u));

A0(eq_37,k_t_f) = 1;
A0(eq_37,i_t_f) = -i_k;
A0(eq_37,eps_i_t) = -i_k*gamI*(gz^2)*(1+beta);
A1(eq_37,k_t_f) = (1-delta)/gz;

A0(eq_38,c_rt_t_f) = c_rt;
A0(eq_38,w_t_f) = -wh_c;
A0(eq_38,h_t_f) = -wh_c;

A0(eq_39,c_t_f) = c_y;
A0(eq_39,g_t) = 1;
A0(eq_39,i_t_f) = i_y;
A0(eq_39,y_t_f) = -1;
A0(eq_39,u_t_f) = (rk/gz)*k_y;

A0(eq_40,c_t_f) = 1;
A0(eq_40,c_rt_t_f) = -theta*c_rt;
A0(eq_40,c_o_t_f) = -(1-theta)*c_o;

A0(eq_41,w_t_f) = 1;
A0(eq_41,mrs_o_t_f) = -(1/(1+omega));
A0(eq_41,mrs_rt_t_f) = -(omega/(1+omega));

A0(eq_42,mrs_o_t_f) = 1;
A0(eq_42,c_o_t_f) = -(sigma/(1-b*(1/c_o)));
A0(eq_42,h_t_f) = -phi_l;
A1(eq_42,c_t_f) = -((b*sigma)/(c_o-b));

A0(eq_43,mrs_rt_t_f) = 1;
A0(eq_43,c_rt_t_f) = -(sigma/(1-b*(1/c_rt)));
A0(eq_43,h_t_f) = -phi_l;
A1(eq_43,c_t_f) = -((b*sigma)/(c_rt-b));

A0(eq_44,u_t_f) = 1;
A0(eq_44,h_t_f) = -1;
A0(eq_44,w_t_f) = -1;
A0(eq_44,r_k_t_f) = 1;
A1(eq_44,k_t_f) = -1;

A0(eq_45,eps_a_t) = 1;
A0(eq_45,r_k_t_f) = -alpha;
A0(eq_45,w_t_f) = -(1-alpha);

A0(eq_46,y_t_f) = 1;
A0(eq_46,eps_a_t) = -(1+lambda_p);
A0(eq_46,u_t_f) = -(1+lambda_p)*alpha;
A0(eq_46,h_t_f) = -(1+lambda_p)*(1-alpha);
A1(eq_46,k_t_f) = (1+lambda_p)*alpha;

A0(eq_47,c_o_t_f) = 1;
A1(eq_47,Ec_o_t_f) = 1;
C(eq_47,eco_sh_f) = 1;

A0(eq_48,i_t_f) = 1;
A1(eq_48,Ei_t_f) = 1;
C(eq_48,ei_sh_f) = 1;

A0(eq_49,r_k_t_f) = 1;
A1(eq_49,Er_k_t_f) = 1;
C(eq_49,erk_sh_f) = 1;

A0(eq_50,Q_o_t_f) = 1;
A1(eq_50,EQ_o_t_f) = 1;
C(eq_50,eQ_sh_f) = 1;

A0(eq_51,gc_t) = 1;
A0(eq_51,c_t) = -1;
A1(eq_51,c_t) = -1;

A0(eq_52,gi_t) = 1;
A0(eq_52,i_t) = -1;
A1(eq_52,i_t) = -1;

A0(eq_53,gw_t) = 1;
A0(eq_53,w_t) = -1;
A1(eq_53,w_t) = -1;

%=========================================================================
%         8. Augmented equations from Bianchi Nicolo method 
%=========================================================================


A0(eq_54,omega1_t) = 1;
A1(eq_54,omega1_t) = alpha1;
B(eq_54,v1_sh) = 1;
C(eq_54,epi_sh) = -1;


%=========================================================================
%           QZ(generalized Schur) decomposition by sims_solve
%=========================================================================


[MY,ME,ETA,AMU_,ne,r2,RC]=sims_solve(A0,A1,B,C,1+1e-8);